# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Qt UI."""

from twisted.internet import defer

from ubuntuone.controlpanel.gui.qt import gui
from ubuntuone.controlpanel.gui.qt.tests import BaseTestCase


class MainWindowTestCase(BaseTestCase):
    """Test the qt main window."""

    innerclass_ui = gui.mainwindow_ui
    innerclass_name = "Ui_MainWindow"
    class_ui = gui.MainWindow

    def test_close_event_calls_custom_close_callback(self):
        """When closing the window, close_callback is called."""
        self.ui.close_callback = self._set_called
        self.ui.closeEvent(event=gui.QtGui.QCloseEvent())
        self.assertEqual(self._called,
                         ((), {}), 'close_callback called.')

    def test_close_callback_can_be_none(self):
        """The close_callback can be None."""
        self.ui.close_callback = None
        self.ui.closeEvent(event=gui.QtGui.QCloseEvent())
        # world did not explode

    def test_on_signin_canceled(self):
        """On SigninPanel's signinCanceled, close."""
        self.patch(self.ui, 'closeEvent', self._set_called)
        self.ui.ui.control_panel.ui.signin.signinCanceled.emit()
        self.assertEqual(len(self._called[0]), 1)
        self.assertIsInstance(self._called[0][0], gui.QtGui.QCloseEvent)


class FakeMainWindow(object):
    """Fake window."""

    def __init__(self, close_callback=None):
        """Create a new instance."""
        self.called = []
        self.close_callback = None

    def __call__(self, *args, **kwargs):
        """Make the object callable to fake a constructor."""
        return self

    def show(self):
        """Fake show the window."""
        self.called.append(('show',))


class FakeTrayIcon(object):
    """Fake tray icon."""

    def __init__(self, window=None):
        """Create a new instance."""
        self.auto_update = None
        self.window = None

    def __call__(self, window, *args, **kwargs):
        """Make the object callable to fake a cosntructor."""
        self.window = window
        return self

    def get_auto_update_lc(self):
        """Get the auto_update_lc."""
        return self.auto_update

    def set_auto_update_lc(self, value):
        """Set the auto_update_lc."""
        self.auto_update = value

    auto_update_lc = property(get_auto_update_lc,
                              set_auto_update_lc)


class FakeLoopingCall(object):
    """Fake a looping call."""

    def __init__(self):
        """Create a new instance."""
        self.function = None
        self.args = None
        self.kwargs = None
        self.called = []

    def __call__(self, f, *args, **kwargs):
        """Call looping call."""
        self.function = f
        self.args = args
        self.kwargs = kwargs
        return self

    def start(self, interval, now=False):
        """Fake the start."""
        self.called.append(('start', interval, now))


class StartTestCase(BaseTestCase):
    """Test the start method."""

    @defer.inlineCallbacks
    def setUp(self):
        """Set the tests."""
        yield super(StartTestCase, self).setUp()
        self.called = []
        self.auto_update_interval = 0.5
        self.looping_call = FakeLoopingCall()

        def stop():
            """Fake stop function."""
            self.called.append(('stop',))

        def auto_update(title, message, icon, logger):
            """Fake auto_update_cb."""
            self.called.append(('auto_update_cb', title, message,
                                icon, logger))

        self.stop_cb = stop
        self.auto_update_cb = auto_update
        self.main_window = FakeMainWindow()
        self.patch(gui, 'MainWindow', self.main_window)
        self.tray_icon = FakeTrayIcon()
        self.patch(gui, 'TrayIcon', self.tray_icon)
        self.patch(gui, 'LoopingCall', self.looping_call)

    def test_start_not_minimized_no_icon(self):
        """Test the not minimized mode with no icon."""
        icon, window = gui.start(self.stop_cb, minimized=False,
                          with_icon=False,
                          auto_update_cb=self.auto_update_cb,
                          auto_update_interval=self.auto_update_interval)
        self.assertEqual(1, len(self.main_window.called))
        self.assertTrue('show' in self.main_window.called[0])
        self.assertEqual(None, self.looping_call.function)
        self.assertNotEqual(None, window)
        self.assertEqual(None, icon)

    def _assert_minimized_or_icon(self, icon, window):
        """Assert the start of a minimized of icon mode."""
        self.assertIsInstance(self.tray_icon.auto_update, FakeLoopingCall)
        self.assertNotEqual(None, self.looping_call.function)
        self.assertNotEqual(None, icon)

    def test_start_with_icon(self):
        """Test the with icon start."""
        icon, window = gui.start(self.stop_cb, minimized=False,
                          with_icon=True,
                          auto_update_cb=self.auto_update_cb,
                          auto_update_interval=self.auto_update_interval)
        self._assert_minimized_or_icon(icon, window)
        self.assertEqual(1, len(self.main_window.called))
        self.assertTrue('show' in self.main_window.called[0])
        self.assertNotEqual(None, window)
        self.assertNotEqual(None, self.tray_icon.window)

    def test_start_minimized(self):
        """Test the minimized start."""
        icon, window = gui.start(self.stop_cb, minimized=True,
                          with_icon=False,
                          auto_update_cb=self.auto_update_cb,
                          auto_update_interval=self.auto_update_interval)
        self._assert_minimized_or_icon(icon, window)
        self.assertEqual(0, len(self.main_window.called))
        self.assertEqual(None, window)
        self.assertEqual(None, self.tray_icon.window)
