# -*- coding: utf-8 -*-

# Author: Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the notification area icon."""

from PyQt4 import QtGui
from twisted.internet import defer, reactor

from ubuntuone.controlpanel.gui.qt import systray
from ubuntuone.controlpanel.tests import TestCase
import ubuntuone.controlpanel.gui.qt.gui


class FakeSDTool(object):

    """Fake SyncDaemonTool."""

    called = False

    def quit(self):
        """Fake quit."""
        self.called = True


class FakeMainWindow(QtGui.QWidget):

    """Fake Main Window."""

    def __init__(self, *args, **kwargs):
        self.args = (args, kwargs)
        super(FakeMainWindow, self).__init__()


class FakeLoopingCall(object):
    """Fake a twisted looping call."""

    def __init__(self, is_running=False):
        """Create a new instance."""
        self.called = []
        self.running = is_running

    def start(self, interval, now=False):
        """Fake starting the call."""
        self.called.append(('start', interval, now))
        self.running = True

    def stop(self):
        """Fake stopping the call."""
        self.called.append(('stop',))
        self.running = False


class SystrayTestCase(TestCase):

    """Test the notification area icon."""

    @defer.inlineCallbacks
    def setUp(self):
        """Set the different tests."""
        yield super(SystrayTestCase, self).setUp()
        self.looping_call = FakeLoopingCall()

    def test_quit_no_lc(self):
        """Test the quit option with no looping call."""
        # Not done on setup, because if I patch stop
        # after instantiation, it doesn't get called.
        self.patch(systray.TrayIcon, "stop", self._set_called)
        tray = systray.TrayIcon()
        tray.quit.trigger()
        self.assertEqual(self._called, ((False,), {}))

    @defer.inlineCallbacks
    def test_quit_lc_not_running(self):
        """Test the quit option with a stopped looping call."""
        st = FakeSDTool()
        self.patch(systray, "SyncDaemonTool", lambda: st)
        self.patch(reactor, 'stop', self._set_called)
        tray = systray.TrayIcon()
        tray.auto_update_lc = self.looping_call
        yield tray.stop()
        self.assertEqual(self._called, ((), {}))
        self.assertEqual(0, len(self.looping_call.called))

    @defer.inlineCallbacks
    def test_quit_lc_running(self):
        """Test the quit option with a running looping call."""
        st = FakeSDTool()
        self.patch(systray, "SyncDaemonTool", lambda: st)
        self.patch(reactor, 'stop', self._set_called)
        self.looping_call.running = True
        tray = systray.TrayIcon()
        tray.auto_update_lc = self.looping_call
        yield tray.stop()
        self.assertEqual(self._called, ((), {}))
        self.assertEqual(1, len(self.looping_call.called))
        self.assertTrue('stop' in self.looping_call.called[0])

    @defer.inlineCallbacks
    def test_stop_sd(self):
        """Quit should call SyncDaemonTool.quit()."""
        st = FakeSDTool()
        self.patch(systray, "SyncDaemonTool", lambda: st)
        self.patch(reactor, "stop", lambda: None)
        tray = systray.TrayIcon()
        yield tray.stop()
        self.assertTrue(st.called)

    def test_restore_no_window(self):
        """Test the restore window option in the menu, with no window."""
        self.patch(ubuntuone.controlpanel.gui.qt.gui,
            "MainWindow", FakeMainWindow)
        tray = systray.TrayIcon()
        self.assertEqual(tray.window, None)
        tray.restore.trigger()
        self.assertIsInstance(tray.window, FakeMainWindow)
        self.assertTrue(tray.window.isVisible())
        self.assertEqual(tray.window.args, ((),
            {'close_callback': tray.delete_window}))

    def test_activate(self):
        """Test the icon activation."""
        tray = systray.TrayIcon()
        window = FakeMainWindow()
        tray.window = window
        self.assertFalse(tray.window.isVisible())
        tray.activated.emit(tray.Trigger)
        self.assertEqual(tray.window, window)
        self.assertTrue(tray.window.isVisible())

    def test_restore_window(self):
        """Test the restore window option in the menu, with a window."""
        tray = systray.TrayIcon()
        window = FakeMainWindow()
        tray.window = window
        self.assertFalse(tray.window.isVisible())
        tray.restore.trigger()
        self.assertEqual(tray.window, window)
        self.assertTrue(tray.window.isVisible())

    def test_delete_window(self):
        """Test deleting an existing window."""
        tray = systray.TrayIcon()
        window = FakeMainWindow()
        tray.window = window
        tray.delete_window()
        self.assertEqual(tray.window, None)
        self.assertFalse(window.isVisible())

    def test_delete_no_window(self):
        """Test deleting without an existing window."""
        tray = systray.TrayIcon()
        tray.delete_window()
        self.assertEqual(tray.window, None)

    def test_initialization(self):
        """Test that everything initializes correctly."""
        tray = systray.TrayIcon()
        self.assertTrue(tray.isVisible())
        self.assertEqual(tray.window, None)
        self.assertIsInstance(tray.context_menu, QtGui.QMenu)
        self.assertFalse(tray.icon() == None)

    def test_show_message(self):
        """Test the method that shows a message."""
        called = []
        title = 'updates'
        message = 'you want to update!'

        def fake_qt_show_message(instance, title, message):
            """A fake qt show message."""
            called.extend([instance, title, message])

        def fake_clicked_message_cb():
            """A fake clicked cb."""

        self.patch(systray.TrayIcon, 'showMessage', fake_qt_show_message)
        tray = systray.TrayIcon()
        tray.show_message(title, message, fake_clicked_message_cb)
        self.assertTrue(tray in called)
        self.assertTrue(title in called)
        self.assertTrue(message in called)
        # pylint: disable=W0212
        self.assertEqual(fake_clicked_message_cb, tray._message_cb)
        # pylint: enable=W0212
