# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Integration tests for the libsoup based webservice client."""

import time

from twisted.application import internet, service
from twisted.internet import defer
from twisted.web import http, resource, server

from ubuntuone.controlpanel.tests import TestCase
from ubuntuone.controlpanel.web_client.libsoup import LibsoupTimestampChecker


class MockResource(resource.Resource):
    """A mock resource."""

    isLeaf = True

    def __init__(self):
        """Initialize this mock instance."""
        resource.Resource.__init__(self)
        self.request_headers = []

    # pylint: disable=C0103
    def render_GET(self, request):
        """Render some content."""
        self.request_headers.append(request.requestHeaders)
        return "hello!"


class MockWebService(object):
    """A mock webservice for testing"""

    def __init__(self):
        """Start up this instance."""
        self.root = MockResource()
        site = server.Site(self.root)
        application = service.Application('web')
        self.service_collection = service.IServiceCollection(application)
        #pylint: disable=E1101
        self.tcpserver = internet.TCPServer(0, site)
        self.tcpserver.setServiceParent(self.service_collection)
        self.service_collection.startService()

    def get_url(self):
        """Build the url for this mock server."""
        #pylint: disable=W0212
        port_num = self.tcpserver._port.getHost().port
        return "http://localhost:%d/" % port_num

    def stop(self):
        """Shut down the service."""
        #pylint: disable=E1101
        self.service_collection.stopService()


class LibsoupTimestampCheckerTestCase(TestCase):
    """Tests for LibsoupTimestampChecker."""

    timeout = 3

    @defer.inlineCallbacks
    def setUp(self):
        yield super(LibsoupTimestampCheckerTestCase, self).setUp()
        self.ws = MockWebService()
        self.addCleanup(self.ws.stop)

    @defer.inlineCallbacks
    def test_gets_server_date(self):
        """The server date is gotten right."""
        fake_time = 1
        self.patch(time, "time", lambda: fake_time)
        checker = LibsoupTimestampChecker()
        self.addCleanup(checker.shutdown)
        d = checker.get_server_date_header(self.ws.get_url())
        result = yield d
        result_time = http.stringToDatetime(result)
        self.assertEqual(result_time, fake_time)

    @defer.inlineCallbacks
    def test_server_date_sends_nocache_headers(self):
        """Getting the server date sends the no-cache headers."""
        checker = LibsoupTimestampChecker()
        self.addCleanup(checker.shutdown)
        yield checker.get_server_date_header(self.ws.get_url())
        self.assertEqual(len(self.ws.root.request_headers), 1)
        headers = self.ws.root.request_headers[0]
        result = headers.getRawHeaders("Cache-Control")
        self.assertEqual(result, ["no-cache"])
